"""Includes DqTestType class."""


class DqTestType:
    """Dq test types"""
    TABLE_NEW_FIELD_CONTROL = 1
    TABLE_REMOVED_FILED_CONTROL = 2
    TABLE_FILED_TYPE_CONTROL = 3
    TABLE_FILED_LENGTH_CONTROL = 4
    TABLE_EMPTY_TABLE_CONTROL = 5
    TABLE_SIZE_CI_CONTROL = 6
    TABLE_SIZE_MODELLING_AND_CI_CONTROL = 7
    TABLE_SIZE_MODELLING_AND_TL_CONTROL = 8
    TABLE_SIZE_MODELLING_AND_AD_CONTROL = 9
    TABLE_COUNT_CI_CONTROL = 10
    TABLE_COUNT_MODELLING_AND_CI_CONTROL = 11
    TABLE_COUNT_MODELLING_AND_TL_CONTROL = 12
    TABLE_COUNT_MODELLING_AND_AD_CONTROL = 13
    TABLE_SIZE_TL_CONTROL = 14
    TABLE_COUNT_TL_CONTROL = 15
    PSI = 16
    TRAFFIC_LIGHT = 17
    UNEXPECTED_VALUE_CONTROL = 18
    MEAN_TL = 19
    MEAN_CI = 20
    MEAN_AD = 21
    MODELLING_AND_TL = 22
    MODELLING_AND_CI = 23
    COLUMN_RANGE_CONTROL = 24
    COLUMN_POPULATION_PERCENT_CONTROL = 25
    COLUMN_VALID_VALUE_CONTROL = 26
    COLUMN_UNVALID_VALUE_CONTROL = 27
    CARDINALITY_CONTROL = 28
    ALL_VALUES_ARE_ZERO_CONTROL = 29
    ALL_VALUES_ARE_NULL_CONTROL = 30
