"""todo"""
from enum import Enum


class DqTestGroupType(Enum):
    """todo"""
    TABLE_SCHEMA_CONTROLS = 1
    RULE_BASED_CONTROLS_TABLE_SET = 2
    MODELLING_AND_CI_CONTROLS_TABLE_SET = 3
    TRAFFIC_LIGHT_CONTROLS = 4
    DISTRIBUTIONAL_COMPARISONS = 5
    MODELLING_AND_CI_CONTROLS_COLUMN_SET = 6
    RULE_BASED_CONTROLS_COLUMN_SET = 7
