"""Includes StabilityFeatureReduction class."""
from organon.ml.feature_reduction.settings.objects.base_feature_reduction_settings import BaseFeatureReductionSettings

class StabilityFeatureReductionSettings(BaseFeatureReductionSettings):
    """Settings for stability feature reduction"""
