"""Includes BaseUnsupervisedFeatureSelectionSettings class"""
from organon.ml.feature_selection.domain.objects.settings.base_feature_selection_settings import \
    BaseFeatureSelectionSettings


class BaseUnsupervisedFeatureSelectionSettings(BaseFeatureSelectionSettings):
    """Base settings for unsupervised feature selection services"""
